#include <stdlib.h>
#include "clfe.h"

using namespace win32fe;

cl::cl() {
  compileoutflag = "-Fo";
  linkoutflag    = "-Fe";

  OutputFlag = compilearg.end();
}

int cl::Parse(void) {
  linkarg.push_front("-link");
  if (!compiler::Parse()) {
    if (!verbose) {
      compilearg.push_back("-nologo");
    }
  } else {
    return(-1);
  }
  return(0);
}

void cl::FindInstallation(void) {
  tool::FindInstallation();
  std::string::size_type n = InstallDir.length()-1;
  VisualStudioDir = InstallDir.substr(0,n);
  n = VisualStudioDir.rfind("\\");
  VisualStudioDir = VisualStudioDir.substr(0,n+1);
  if (verbose) {
    std::string longpath;
    GetLongPath(VisualStudioDir,longpath);
    std::cout << "win32fe: Visual Studio Installation: " << longpath << std::endl;
  }
  VSVersion = InstallDir.substr(0,InstallDir.length()-1);
  VSVersion = VSVersion.substr(VisualStudioDir.length());
  if (verbose) {
    std::cout << "win32fe: Visual C++ Version: " << VSVersion << std::endl;
  }
}

void cl::AddPaths(void) {
  /* Find required .dll's */
  std::string addpath;
  bool KnownVersion=false;
  if (VSVersion=="VC98") {
    addpath = VisualStudioDir + "Common\\MSDev98\\Bin";
    KnownVersion=true;
  } else if (VSVersion=="VC7") {
    addpath = VisualStudioDir + "Common7\\IDE";
    KnownVersion=true;
  } else {
    if (!woff) {
      std::cerr << "Warning: win32fe Visual Studio version not recognized." << std::endl;
    }
  }
  if (KnownVersion) {
    arg.push_back("--path");
    LI i = arg.end();
    i--;
    GetShortPath(addpath);
    arg.push_back(addpath);
    FoundPath(i);
  }
}

void cl::Help(void) {
  compiler::Help();
  std::string help = compilearg.front();
  help += " -? 2>&1";
  system(help.c_str());
}

void cl::FoundL(LI &i) {
  std::string temp = i->substr(2);
  ReplaceSlashWithBackslash(temp);
  if (GetShortPath(temp)) {
    temp = "-libpath:"+temp;
    linkarg.push_back(temp);
  } else {
    if (!woff) {
      std::cerr << "Warning: win32fe Library Path Not Found: " << i->substr(2) << std::endl;
    }
  }
}

void df::Help(void) {
  compiler::Help();
  std::cout << "df specific help:" << std::endl;
  std::cout << "  For mixed language C/FORTRAN programming in PETSc, the location of the" << std::endl;
  std::cout << "      C system libraries must also be specified in the LIB environment" << std::endl;
  std::cout << "      variable or with -L<dir> since the installed location of these" << std::endl;
  std::cout << "      libraries may be independent of the FORTRAN installation." << std::endl;
  std::cout << "      If installed, the Visual Studio 6, C system libraries are" << std::endl;
  std::cout << "      automatically found and added to the path." << std::endl << std::endl;
  std::cout << "=========================================================================" << std::endl << std::endl;
  std::string help = compilearg.front();
  help += " -? 2>&1";
  system(help.c_str());
}

void df::AddPaths(void) {
  std::string addpath1,addpath2,DFVersion;
  addpath1 = DFVersion = InstallDir.substr(0,InstallDir.length()-1);
  DFVersion = DFVersion.substr(DFVersion.rfind("\\")+1);
  addpath2 = addpath1 = addpath1.substr(0,addpath1.rfind("\\")+1);

  if (DFVersion=="DF98") {
    addpath1 += "Common\\MSDev98\\Bin";
    addpath2 += "VC98\\Bin";
    GetShortPath(addpath1);
    GetShortPath(addpath2);
    std::string addpath = addpath1 + ";" + addpath2;
    arg.push_back("--path");
    LI i = arg.end();
    i--;
    arg.push_back(addpath);
    FoundPath(i);
  } else {
    if (!woff) {
      std::cerr << "Warning: win32fe df version not recognized." << std::endl;
    }
  }
}

void df::AddSystemLib(void) {
  compiler::AddSystemLib();
  std::string::size_type len_m_1 = InstallDir.length()-1;
  std::string libdir,DFVersion;
  libdir = DFVersion = InstallDir.substr(0,len_m_1);
  DFVersion = DFVersion.substr(DFVersion.rfind("\\")+1,len_m_1);
  libdir = libdir.substr(0,libdir.rfind("\\")+1);

  if (DFVersion=="DF98") {
    libdir += "VC98\\Lib";
    GetShortPath(libdir);
    if (verbose) {
      std::string longpath;
      GetLongPath(libdir,longpath);
      std::cout << "win32fe: Adding Flag: -L" << longpath << std::endl;
    }
    libdir = "-L" + libdir;
    LI i = arg.end();
    arg.push_back(libdir);
    i--;
    FoundL(i);
  } else {
    if (!woff) {
      std::cerr << "Warning: win32fe df version not recognized." << std::endl;
    }
  }
}
    
