#include <Windows.h>
#include "tlibfe.h"

using namespace win32fe;

int tlib::Archive() {
  /* archiver::Execute() strips quotes from archivename, so put them back. */
  std::string archivename = archivearg.back();
  archivearg.pop_back();
  archivename = "\"" + archivename + "\"";
  archivearg.push_back(archivename);

  int ierr = archiver::Archive();

  /* On updates, tlib creates a backup library with .BAK */
  /* Check for existance and remove the .BAK library */
  archivename = archivename.substr(1,archivename.rfind(".")-1);
  archivename = archivename + ".BAK";
  std::string temp=archivename;
  if (GetShortPath(temp)) {
    if (verbose) {
      std::cout << "del " << archivename << std::endl;
    }
    DeleteFile(archivename.c_str()); /* Windows.h */
  }
  return(ierr);
}

void tlib::Help(void) {
  archiver::Help();
  std::cout << "tlib specific help:" << std::endl;
  std::cout << "  win32fe requires the use of - to denote flags instead of /." << std::endl;
  std::cout << "        The translation from - to / for tlib is managed automatically." << std::endl;
  std::cout << "  Note: tlib operators +, -+, *, etc. are not supported by win32fe." << std::endl;
  std::cout << "        Instead, use the arguments -a, -u, -e, etc. accordingly." << std::endl << std::endl;
  std::cout << "=========================================================================" << std::endl << std::endl;
  std::string help = archivearg.front();
  system(help.c_str());
}

void tlib::FoundFlag(LI &i) {
  std::string temp = *i;
  if (temp == "-help") {
    helpfound = TRUE;
  } else {
    temp[0] = '/';
    archivearg.push_back(temp);
  }
}

void tlib::FoundFile(LI &i) {
  tool::FoundFile(i);
  std::string temp=file.back();
  file.pop_back();
  file.push_back("\"" + temp + "\"");
}

/* Note: the following was taken almost verbatim from void bcc32::FindInstallation(void). */
/*       Perhaps someday a different inheritance model should be used. */
void tlib::FindInstallation(void) {
  bool oldwoff = woff;
  woff = true;
  tool::FindInstallation();
  woff = oldwoff;
  if (!GetShortPath(InstallDir)) {
    std::string trypath[3];
    trypath[0]="C:\\Borland\\BCC55\\Bin\\tlib.exe";
    trypath[1]="C:\\Program Files\\Borland\\BCC55\\Bin\\tlib.exe";
    trypath[2]="/software/Borland/BCC55/Bin/tlib.exe";
    for (int i=0;i<3;i++) {
      if (!GetShortPath(trypath[i])) {
        InstallDir = "";
      } else {
        InstallDir = trypath[i].substr(0,trypath[i].rfind("\\"));
        InstallDir = InstallDir.substr(0,InstallDir.rfind("\\")+1);
        break;
      }
    }
  }
  if (InstallDir!="") {
    LI FoundPathHere=arg.end();
    arg.push_back("--path");
    FoundPathHere--;
    arg.push_back(InstallDir+"Bin");
    tool::FoundPath(FoundPathHere);
  } else {
    std::cerr << "Error: tlib.exe installation not found." << std::endl;
    std::cerr << "  Specify the complete path to tlib.exe with --use or --path" << std::endl;
    std::cerr << "  Use --help for more information on win32fe options." << std::endl << std::endl;
  }
}

